@@ -1,6 +1,18 @@
 import unittest
 
-class TestSimpleCleanup(unittest.TestCase):
+class _Base(unittest.TestCase):
+
+    def failIf(self, expr, msg=None):
+        # silence stupid 2.7 stdlib deprecation
+        if expr: #pragma NO COVERAGE
+            raise self.failureException, msg
+
+    def failUnless(self, expr, msg=None):
+        # silence stupid 2.7 stdlib deprecation
+        if not expr: #pragma NO COVERAGE
+            raise self.failureException, msg
+
+class TestSimpleCleanup(_Base):
 
     def _getTargetClass(self):
         from repoze.zodbconn.finder import SimpleCleanup
@@ -17,7 +29,7 @@ class TestSimpleCleanup(unittest.TestCase):
         del cleanup
         self.failUnless(root.closed)
 
-class TestLoggingCleanup(unittest.TestCase):
+class TestLoggingCleanup(_Base):
 
     def _getTargetClass(self):
         from repoze.zodbconn.finder import LoggingCleanup
@@ -57,7 +69,7 @@ class TestLoggingCleanup(unittest.TestCase):
 
 _marker = object()
 
-class TestPersistentApplicationFinder(unittest.TestCase):
+class TestPersistentApplicationFinder(_Base):
     def setUp(self):
         from repoze.zodbconn.resolvers import RESOLVERS
         self.root = DummyRoot()
@@ -80,14 +92,14 @@ class TestPersistentApplicationFinder(unittest.TestCase):
 
     def test_ctor(self):
         from repoze.zodbconn.finder import SimpleCleanup
-        def makeapp(root):
-            pass
+        def makeapp(root): pass
         finder = self._makeOne('foo://bar.baz', makeapp)
         self.assertEqual(finder.uri, 'foo://bar.baz')
         self.assertEqual(finder.appmaker, makeapp)
         self.failUnless(finder.cleanup is SimpleCleanup)
 
     def test_call_no_db_no_cleanup(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
         def makeapp(root):
             root.made = True
             return 'abc'
@@ -95,13 +107,14 @@ class TestPersistentApplicationFinder(unittest.TestCase):
         environ = {}
         app = finder(environ)
         self.assertEqual(app, 'abc')
-        self.assertEqual(self.root.made, True)
-        self.assertEqual(self.root.closed, False)
-        del environ['repoze.zodbconn.closer']
-        self.assertEqual(self.root.closed, True)
+        self.failUnless(self.root.made)
+        self.failIf(self.root.closed)
+        del environ[CLOSER_KEY]
+        self.failUnless(self.root.closed)
         self.assertEqual(finder.db, self.db)
 
     def test_call_no_db_w_cleanup(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
         def makeapp(root):
             root.made = True
             return 'abc'
@@ -109,84 +122,109 @@ class TestPersistentApplicationFinder(unittest.TestCase):
         environ = {}
         app = finder(environ)
         self.assertEqual(app, 'abc')
-        self.assertEqual(self.root.made, True)
-        self.assertEqual(self.root.closed, False)
+        self.failUnless(self.root.made)
+        self.failIf(self.root.closed)
         self.assertEqual(environ['XXX'], None)
-        del environ['repoze.zodbconn.closer']
-        self.assertEqual(self.root.closed, True)
+        del environ[CLOSER_KEY]
+        self.failUnless(self.root.closed)
         self.assertEqual(finder.db, self.db)
 
     def test_call_with_db_no_cleanup(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
         def makeapp(root):
             root.made = True
             return 'abc'
-        finder = self._makeOne('another://bar.baz', makeapp)
+        finder = self._makeOne(None, makeapp)
         finder.db = DummyDB(self.root, 'another')
         environ = {}
         app = finder(environ)
         self.assertEqual(app, 'abc')
-        self.assertEqual(self.root.made, True)
-        self.assertEqual(self.root.closed, False)
-        del environ['repoze.zodbconn.closer']
-        self.assertEqual(self.root.closed, True)
+        self.failUnless(self.root.made)
+        self.failIf(self.root.closed)
+        del environ[CLOSER_KEY]
+        self.failUnless(self.root.closed)
 
     def test_call_with_db_w_cleanup(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
         def makeapp(root):
             root.made = True
             return 'abc'
-        finder = self._makeOne('another://bar.baz', makeapp,
-            cleanup=DummyCleanup)
+        finder = self._makeOne(None, makeapp, cleanup=DummyCleanup)
         finder.db = DummyDB(self.root, 'another')
         environ = {}
         app = finder(environ)
         self.assertEqual(app, 'abc')
-        self.assertEqual(self.root.made, True)
-        self.assertEqual(self.root.closed, False)
+        self.failUnless(self.root.made)
+        self.failIf(self.root.closed)
         self.assertEqual(environ['XXX'], None)
-        del environ['repoze.zodbconn.closer']
-        self.assertEqual(self.root.closed, True)
+        del environ[CLOSER_KEY]
+        self.failUnless(self.root.closed)
 
-    def test_get_connection_from_environ(self):
+    def test_get_connection_from_environ_connector_key(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
+        from repoze.zodbconn.connector import CONNECTION_KEY
         def makeapp(root):
             root.made = True
             return 'abc'
         finder = self._makeOne('foo://bar.baz', makeapp)
         db = DummyDB(self.root, 'another')
         conn = db.open()
-        environ = {'repoze.zodbconn.connection': conn}
+        environ = {CONNECTION_KEY: conn}
         app = finder(environ)
         self.assertEqual(app, 'abc')
-        self.assertEqual(self.root.made, True)
-        self.assertEqual(self.root.closed, False)
-        self.assertFalse('repoze.zodbconn.closer' in environ)
+        self.failUnless(self.root.made)
+        self.failIf(self.root.closed)
+        self.failIf(CLOSER_KEY in environ)
+
+    def test_get_connection_from_environ_closer_key(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
+        def makeapp(root):
+            root.made = True
+            return 'abc'
+        finder = self._makeOne('foo://bar.baz', makeapp)
+        root = DummyRoot()
+        db = DummyDB(root, 'another')
+        conn = db.open()
+        environ = {CLOSER_KEY: conn.close}
+        app = finder(environ)
+        self.assertEqual(app, 'abc')
+        self.failIf('made' in self.root.__dict__)
+        self.failUnless(root.made)
+        self.failIf(self.root.closed)
+        self.failIf(root.closed)
+        self.failUnless(CLOSER_KEY in environ)
 
     def test_ignore_connection_from_environ(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
+        from repoze.zodbconn.connector import CONNECTION_KEY
         def makeapp(root):
             root.made = True
             return 'abc'
         finder = self._makeOne('foo://bar.baz', makeapp, connection_key=None)
         db = DummyDB(self.root, 'another')
         conn = db.open()
-        environ = {'repoze.zodbconn.connection': conn}
+        environ = {CONNECTION_KEY: conn}
         app = finder(environ)
         self.assertEqual(app, 'abc')
-        self.assertEqual(self.root.made, True)
-        self.assertEqual(self.root.closed, False)
-        self.assertTrue('repoze.zodbconn.closer' in environ)
+        self.failUnless(self.root.made)
+        self.failIf(self.root.closed)
+        self.failUnless(CLOSER_KEY in environ)
 
     def test_call_no_uri(self):
+        from repoze.zodbconn.finder import CLOSER_KEY
+        from repoze.zodbconn.connector import CONNECTION_KEY
         def makeapp(root):
             root.made = True
             return 'abc'
         finder = self._makeOne('', makeapp)
         db = DummyDB(self.root, 'another')
         conn = db.open()
-        environ = {'repoze.zodbconn.connection': conn}
+        environ = {CONNECTION_KEY: conn}
         app = finder(environ)
         self.assertEqual(app, 'abc')
-        self.assertEqual(self.root.made, True)
-        self.assertEqual(self.root.closed, False)
-        self.assertFalse('repoze.zodbconn.closer' in environ)
+        self.failUnless(self.root.made)
+        self.failIf(self.root.closed)
+        self.failIf(CLOSER_KEY in environ)
 
 
 class DummyRoot:
@@ -229,4 +267,6 @@ class DummyLogger:
         self._wrote = []
 
     def write(self, chunk):
-        self._wrote.append(chunk)
+        # This line *does* get called, but somehow the coverage plugin
+        # misses it (maybe because it is inside the closer's __del__?
+        self._wrote.append(chunk) #pragma NO COVERAGE
